package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.TmstruckTimeTaskResultMapper;
import com.steerinfo.dil.mapper.UtilsMapper;
import com.steerinfo.dil.model.TmstruckTimeTaskResult;
import com.steerinfo.dil.service.ITmstruckTimeTaskResultService;

import com.steerinfo.dil.util.DataChange;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * TmstruckTimeTskResult服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-10-22 07:28
 * 类描述
 * 修订历史：
 * 日期：2021-10-22
 * 作者：generator
 * 参考：
 * 描述：TmstruckTimeTskResult服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmstruckTimeTaskResultService")
public class TmstruckTimeTaskResultServiceImpl implements ITmstruckTimeTaskResultService {

    @Autowired
    private TmstruckTimeTaskResultMapper tmstruckTimeTaskResultMapper;
    @Autowired
    private UtilsMapper utilsMapper;
    @Autowired
    private UtilsServiceImpl utilsService;


    /**
     * 通用生成实例数据
     * @param map
     * @return
     */
    public TmstruckTimeTaskResult generateTimeTaskResult(Map<String, Object> map, Integer timeType){
        TmstruckTimeTaskResult tmstruckTimeTaskResult = new TmstruckTimeTaskResult();
        //获取主键ID
        tmstruckTimeTaskResult.setId(tmstruckTimeTaskResultMapper.selectMaxId());
        String orderNumber = (String) map.get("orderNumber");
        //通过运输订单号查询总实绩ID等数据
        map.putAll(tmstruckTimeTaskResultMapper.getResultTotalIdByOrderNum(orderNumber));
        tmstruckTimeTaskResult.setResultTotalId(DataChange.dataToBigDecimal(map.get("resultTotalId")));
        //生成时间
        tmstruckTimeTaskResult.setResultTime(new Date());
        //设置时间类型
        tmstruckTimeTaskResult.setResultTimeType(new BigDecimal(timeType));
        return tmstruckTimeTaskResult;
    }

    /**
     * 司机申请开始
     * @param map
     * @return
     */
    @Override
    public int applyStartTime(Map<String, Object> map) {
        //设置时间类型 0
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 0);
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }

    /**
     * 用车单位确认开始
     * @param map
     * @return
     */
    @Override
    public int unitSureStartTime(Map<String, Object> map) {
        //设置时间类型 1
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 1);
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }

    /**
     * 司机申请暂停
     * @param map
     * @return
     */
    @Override
    public int applyStartTimeout(Map<String, Object> map) {
        //设置时间类型 4
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 4);
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }

    /**
     * 用车单位确认
     * @param map
     * @return
     */
    @Override
    public int unitSureStartTimeout(Map<String, Object> map) {
        //设置时间类型 5
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 5);
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }

    /**
     * 司机申请暂停结束
     * @param map
     * @return
     */
    @Override
    public int applyEndTimeout(Map<String, Object> map) {
        //设置时间类型 6
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 6);
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }

    /**
     * 用车单位确认暂停结束
     * @param map
     * @return
     */
    @Override
    public int unitSureEndTimeout(Map<String, Object> map) {
        //设置时间类型 7
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 7);
        //查询上一次结束的时间
        Map<String, Object> timeMap = tmstruckTimeTaskResultMapper.getLastTimeout(tmstruckTimeTaskResult.getResultTotalId());
        Object resultTime = timeMap.get("resultTime");
        if(resultTime != null){
            Date lastTimeout = (Date) resultTime;
            //计算相差时间戳 存入当前实绩中  更精确
            long differenceTime = new Date().getTime() - lastTimeout.getTime();
            tmstruckTimeTaskResult.setResultValue(new BigDecimal(differenceTime));
        }
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }

    /**
     * 司机申请结束
     * @param map
     * @return
     */
    @Override
    public int applyEndTime(Map<String, Object> map) {
        //设置时间类型 2
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 2);
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }

    /**
     * 用车单位确认结束
     * @param map
     * @return
     */
    @Override
    public int unitSureEndTime(Map<String, Object> map) {
        //设置时间类型 3
        TmstruckTimeTaskResult tmstruckTimeTaskResult = generateTimeTaskResult(map, 3);
        //查询当前车辆排队开始时间 计算出总时长
        Map<String, Object> startTimeMap = tmstruckTimeTaskResultMapper.getStartTime(tmstruckTimeTaskResult.getResultTotalId());
        Date endTime = new Date();
        //计算当前时间与确认开始时间相差的毫秒数（时间戳）
        Date startTime = (Date) startTimeMap.get("resultTime");
        long allTime = endTime.getTime() - startTime.getTime();
        //查询所有确认暂停结束的时长时间戳
        List<Map<String, Object>> allTimeout = tmstruckTimeTaskResultMapper.getAllTimeout(tmstruckTimeTaskResult.getResultTotalId());
        long allTimeoutTime = 0;
        //计算所有暂停时间时长时间戳总和
        for (Map<String, Object> timeoutMap : allTimeout) {
            BigDecimal resultTime = DataChange.dataToBigDecimal(timeoutMap.get("resultValue"));
            allTimeoutTime += resultTime.longValue();
        }
        long realTime = allTime - allTimeoutTime;
        //计算总工作时长（分钟）
        realTime = realTime / (1000 * 60);
        tmstruckTimeTaskResult.setResultValue(new BigDecimal(realTime));
        return tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
    }


    /**
     * 查询计时作业
     * @param map
     * @return
     */
    @Override
    public List<Map<String, Object>> getAllTimeTaskResult(Map<String, Object> map) {
        return tmstruckTimeTaskResultMapper.getAllTimeTaskResult(map);
    }

    /**
     * 添加计时作业实绩
     * @param map
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int addTimeTaskResult(Map<String, Object> map) throws Exception {
        int i=0;
        Date resultTime2 = null;
        Date resultTime1;
        String orderNumber =(String)map.get("orderNumber");
        BigDecimal resultTotalId = utilsMapper.getTotalIdByOrderNumber(orderNumber);
        TmstruckTimeTaskResult tmstruckTimeTaskResult=new TmstruckTimeTaskResult();
        BigDecimal orderId = utilsMapper.getOrderIdByOrderNumber(orderNumber);
        map.put("orderId",orderId);
        //根据运输订单id查找实绩中是否已经含有计时结束的实绩，如果有则抛出异常
        int countEndTimeNum = tmstruckTimeTaskResultMapper.CountTimeTaskByOrderId(orderId);
        if (countEndTimeNum>=1){
            throw new Exception("请勿重复计时");
        }
        //根据运输订单号判断是否计时开始
        BigDecimal Time = utilsMapper.getTime(orderId);
        if (Time==null) {
            tmstruckTimeTaskResult.setTimeTaskResultId(tmstruckTimeTaskResultMapper.selectMaxId());
            tmstruckTimeTaskResult.setResultTotalId(DataChange.dataToBigDecimal(resultTotalId));
            //获取计时开始的订单类型
            BigDecimal inwardType = DataChange.dataToBigDecimal(map.get("inwardType"));
            tmstruckTimeTaskResult.setResultTimeType(inwardType);
            tmstruckTimeTaskResult.setResultTime(new Date());
            tmstruckTimeTaskResult.setInsertTime(new Date());
            //新增第一条计时实绩
            i += tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
        }else {
            //已经有计时开始时间
            //新增第二条实绩，计时结束
            tmstruckTimeTaskResult.setResultTotalId(DataChange.dataToBigDecimal(resultTotalId));
            //获取计时结束的订单类型
            tmstruckTimeTaskResult.setTimeTaskResultId(tmstruckTimeTaskResultMapper.selectMaxId());
            tmstruckTimeTaskResult.setResultTime(new Date());
            tmstruckTimeTaskResult.setResultTimeType(new BigDecimal(3));
            i += tmstruckTimeTaskResultMapper.insertSelective(tmstruckTimeTaskResult);
            //此时获取他的上一条实绩生成计时时间
        }
        //判断如果计时结束则生成作业时长，根据运输订单号查找作业是否结束
        Map<String,Object> timeStatus = utilsMapper.getTimeStatus(map);
        //首先判断计时是否开始,如果计时开始则生成计时结束实绩
        //通过运输订单id和实绩结束状态查找时间实绩id
        resultTime1 =(Date) timeStatus.get("resultTime");
        Map<String, Object> timeTask = utilsMapper.FristTimeTask(map);
        //获取计时开始时间
        if (resultTime1!=null&&timeTask!=null) {//生成计时结束时间得同时.生成作业时长
            BigDecimal timeTaskresultId = DataChange.dataToBigDecimal(timeTask.get("timeResultId"));
            map.put("timeTaskresultId",timeTaskresultId);
            Map<String, Object> timeMaps = utilsMapper.FristTimeTask(map);
            resultTime2 = (Date) timeMaps.get("resultTime");
            BigDecimal timeResultId = (BigDecimal) timeMaps.get("timeResultId");
            long dateTime = resultTime2.getTime() - resultTime1.getTime();
            tmstruckTimeTaskResult.setTimeTaskResultId(timeResultId);
            tmstruckTimeTaskResult.setResultValue(new BigDecimal(dateTime / 3600000));
            tmstruckTimeTaskResult.setResultUomId(new BigDecimal(900));
            tmstruckTimeTaskResult.setUpdateTime(new Date());
             i+=tmstruckTimeTaskResultMapper.updateByPrimaryKeySelective(tmstruckTimeTaskResult);
            map.put("orderStatus", 2);
            utilsMapper.updateOrderStatusByOrderIdOrNum(map);
        }
        return i;
    }


}
